/* Franny 1.0
 * User's application
 * by Rafal BOBER Ciepiela
 * contact: bob_er@interia.pl
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef FRANNY_10
#define FRANNY_10

const char* PROGRAM_NAME="franny";

const int COMMAND_NONE=0;
const int COMMAND_CREATE=1;
const int COMMAND_LIST=2;
const int COMMAND_INFO=3;
const int COMMAND_DUMP=4;
const int COMMAND_SAVE=5;
const int COMMAND_ADD=6;
const int COMMAND_UNLINK=7;
const int COMMAND_TYPE=8;
const int COMMAND_NAME=9;
const int COMMAND_MKDIR=10;
const int COMMAND_RMDIR=11;
const int COMMAND_FORMAT=12;
const int COMMAND_CONV=13;

#endif /* FRANNY_10 */
